package cn.com.ctop.kuaishou.modules.ai.controller;

import cn.com.ctop.common.module.annotation.AutoLog;
import cn.com.ctop.common.module.entity.CtopOauthToken;
import cn.com.ctop.common.module.service.ICtopOauthTokenService;
import cn.com.ctop.common.module.utils.Check;
import cn.com.ctop.common.module.utils.QueryGenerator;
import cn.com.ctop.kuaishou.modules.ai.entity.KuaishouDirectionalPackage;
import cn.com.ctop.kuaishou.modules.ai.service.IKuaishouDirectionalPackageService;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.util.oConvertUtils;
import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * 自动投放-定向包详情
 *
 * @author jeecg-boot
 * @version V1.0
 * @date 2021-04-20
 */
@Slf4j
@Api(tags = "自动投放-定向包详情")
@RestController
@RequestMapping("/kuaishouDirectionalPackage")
public class KuaishouDirectionalPackageController {
    @Autowired
    private IKuaishouDirectionalPackageService kuaishouDirectionalPackageService;

    @Autowired
    private ICtopOauthTokenService oauthTokenService;

    /**
     * 分页列表查询
     *
     * @param kuaishouDirectionalPackage
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @AutoLog(value = "自动投放-定向包详情-分页列表查询")
    @ApiOperation(value = "自动投放-定向包详情-分页列表查询", notes = "自动投放-定向包详情-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<KuaishouDirectionalPackage>> queryPageList(KuaishouDirectionalPackage kuaishouDirectionalPackage,
                                                                   @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                                   @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                                   HttpServletRequest req) {
        Result<IPage<KuaishouDirectionalPackage>> result = new Result<>();
        QueryWrapper<KuaishouDirectionalPackage> queryWrapper = QueryGenerator.initQueryWrapper(kuaishouDirectionalPackage, req.getParameterMap());
        Page<KuaishouDirectionalPackage> page = new Page<KuaishouDirectionalPackage>(pageNo, pageSize);
        IPage<KuaishouDirectionalPackage> pageList = kuaishouDirectionalPackageService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    /**
     * 查询定向包列表
     *
     * @param
     * @return org.jeecg.common.api.vo.Result<java.lang.Object>
     * @throws
     * @author ZHAOXA
     */
    @GetMapping(value = "/queryList")
    public Result<Object> queryList(KuaishouDirectionalPackage directionalPackage,
                                    @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                    @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize) {
        try {
            return kuaishouDirectionalPackageService.queryList(directionalPackage, pageNo, pageSize);
        } catch (Exception e) {
            log.error("查询异常,", e);
            return Result.error("查询失败，" + e.getMessage());
        }

    }

    /**
     * 相关账户/组
     *
     * @param projectId 账户ID
     * @param type      1是账户，2是组维度
     * @return org.jeecg.common.api.vo.Result<java.lang.Object>
     * @throws
     * @author ZHAOXA
     */
    @GetMapping(value = "/getRelatedAccountGroup")
    public Result<Object> getRelatedAccountGroup(String id, Integer type, @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo, @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize) {
        try {
            if (Check.isNull(id)) {
                return Result.error("请选择人群包");
            }
            if (Check.isNull(type)) {
                return Result.error("请选择查询类型");
            }
            return kuaishouDirectionalPackageService.getRelatedAccountGroup(id, type, pageNo, pageSize);
        } catch (Exception e) {
            log.error("查询异常,", e);
            return Result.error("查询失败");
        }

    }

    /**
     * 查询定向包推送历史
     *
     * @param projectId 账户ID
     * @return org.jeecg.common.api.vo.Result<java.lang.Object>
     * @throws
     * @author ZHAOXA
     */
    @GetMapping(value = "/getHistory")
    public Result<Object> getHistory(Long projectId, Long templateId,
                                     @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                     @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize) {
        try {
            if (Check.isNull(templateId)) {
                return Result.error("请选择人群包");
            }
            if (Check.isNull(projectId)) {
                return Result.error("请选择项目");
            }

            return kuaishouDirectionalPackageService.getHistory(projectId, templateId, pageNo, pageSize);
        } catch (Exception e) {
            log.error("查询异常,", e.getMessage());
            e.printStackTrace();
        }
        return Result.error("查询失败");
    }

    /**
     * 查询项目下有效账户
     *
     * @param projectId 账户ID
     * @return org.jeecg.common.api.vo.Result<java.lang.Object>
     * @throws
     * @author ZHAOXA
     */
    @GetMapping(value = "/getAccountByProjectId")
    public Result<Object> getAccountByProjectId(Long projectId,
                                                @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize) {
        try {
            if (Check.isNull(projectId)) {
                return Result.error("请选择项目");
            }
            return kuaishouDirectionalPackageService.getAccountByProjectId(projectId, pageNo, pageSize);
        } catch (Exception e) {
            log.error("查询异常,", e.getMessage());
            e.printStackTrace();
        }
        return Result.error("查询失败");
    }

    /**
     * 创建定向包
     *
     * @param
     * @return
     */
    @PostMapping(value = "/createDirectionalPackage")
    public Result<Object> createDirectionalPackage(@RequestBody JSONObject requestJson) {
        Result<JSONObject> result = new Result<>();
        try {
            if (Check.isNull(requestJson.getString("templateName"))) {
                return Result.error("定向模板名称为必传项");
            }
            Long accountId = requestJson.getLong("accountId");
            if (Check.isNull(accountId)) {
                return Result.error("请选择账户id");
            }
            CtopOauthToken oauthToken = oauthTokenService.getTokenByAccountId(accountId);
            if (Check.isNull(oauthToken)) {
                return Result.error("未获取到账户信息");
            }
            return kuaishouDirectionalPackageService.createDirectionalPackage(oauthToken.getAccessToken(), requestJson);
        } catch (Exception e) {
            log.error("创建定向包失败", e);
            return Result.error("创建失败" + e.getMessage());
        }
    }

    /**
     * 修改定向包
     *
     * @param
     * @return
     */
    @PostMapping(value = "/updateDirectionalPackage")
    public Result<Object> updateDirectionalPackage(@RequestBody JSONObject requestJson) {
        Result<JSONObject> result = new Result<>();
        try {
            if (Check.isNull(requestJson.getString("templateName"))) {
                return Result.error("定向模板名称为必传项");
            }
            if (Check.isNull(requestJson.getLong("templateId"))) {
                return Result.error("定向模板id为必传项");
            }
            Long accountId = requestJson.getLong("accountId");
            if (Check.isNull(accountId)) {
                return Result.error("请选择账户id");
            }
            CtopOauthToken oauthToken = oauthTokenService.getTokenByAccountId(accountId);
            if (Check.isNull(oauthToken)) {
                return Result.error("未获取到账户信息");
            }
            return kuaishouDirectionalPackageService.updateDirectionalPackage(oauthToken.getAccessToken(), requestJson);
        } catch (Exception e) {
            result.setSuccess(false);
            result.setMessage(e.getMessage());
        }
        return Result.error("上传失败");
    }

    /**
     * 推送定向包
     *
     * @param
     * @return
     */
    @PostMapping(value = "/pushDirectionalPackage")
    public Result<Object> pushDirectionalPackage(@RequestBody JSONObject requestJson) {
        Result<JSONObject> result = new Result<>();
        try {
            String id = requestJson.getString("id");
            JSONArray accountIds = requestJson.getJSONArray("accountIds");
            if (Check.isNull(id)) {
                return Result.error("请选择人群包");
            }
            if (Check.isNull(accountIds)) {
                return Result.error("请选择推送账户");
            }
            return kuaishouDirectionalPackageService.pushDirectionalPackage(id, accountIds);
        } catch (Exception e) {
            result.setSuccess(false);
            result.setMessage(e.getMessage());
        }
        return Result.error("上传失败");
    }

    /**
     * 删除定向模板
     *
     * @param
     * @return org.jeecg.common.api.vo.Result<?>
     * @throws
     * @author ZHAOXA
     */
    @GetMapping(value = "/delete")
    public Result<Object> delete(Long accountId, Long templateId) {
        try {
            return kuaishouDirectionalPackageService.delete(accountId, templateId);
        } catch (Exception e) {
            return Result.error("删除失败!");
        }
    }

    /**
     * 添加
     *
     * @param kuaishouDirectionalPackage
     * @return
     */
    @AutoLog(value = "自动投放-定向包详情-添加")
    @ApiOperation(value = "自动投放-定向包详情-添加", notes = "自动投放-定向包详情-添加")
    @PostMapping(value = "/add")
    public Result<KuaishouDirectionalPackage> add(@RequestBody KuaishouDirectionalPackage kuaishouDirectionalPackage) {
        Result<KuaishouDirectionalPackage> result = new Result<>();
        try {
            kuaishouDirectionalPackageService.save(kuaishouDirectionalPackage);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param kuaishouDirectionalPackage
     * @return
     */
    @AutoLog(value = "自动投放-定向包详情-编辑")
    @ApiOperation(value = "自动投放-定向包详情-编辑", notes = "自动投放-定向包详情-编辑")
    @PutMapping(value = "/edit")
    public Result<KuaishouDirectionalPackage> edit(@RequestBody KuaishouDirectionalPackage kuaishouDirectionalPackage) {
        Result<KuaishouDirectionalPackage> result = new Result<KuaishouDirectionalPackage>();
        KuaishouDirectionalPackage kuaishouDirectionalPackageEntity = kuaishouDirectionalPackageService.getById(kuaishouDirectionalPackage.getId());
        if (kuaishouDirectionalPackageEntity == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = kuaishouDirectionalPackageService.updateById(kuaishouDirectionalPackage);
            if (ok) {
                result.success("修改成功!");
            }
        }

        return result;
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "自动投放-定向包详情-批量删除")
    @ApiOperation(value = "自动投放-定向包详情-批量删除", notes = "自动投放-定向包详情-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<KuaishouDirectionalPackage> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<KuaishouDirectionalPackage> result = new Result<>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.kuaishouDirectionalPackageService.removeByIds(Arrays.asList(ids.split(",")));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @AutoLog(value = "自动投放-定向包详情-通过id查询")
    @ApiOperation(value = "自动投放-定向包详情-通过id查询", notes = "自动投放-定向包详情-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<KuaishouDirectionalPackage> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<KuaishouDirectionalPackage> result = new Result<>();
        KuaishouDirectionalPackage kuaishouDirectionalPackage = kuaishouDirectionalPackageService.getById(id);
        if (kuaishouDirectionalPackage == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(kuaishouDirectionalPackage);
            result.setSuccess(true);
        }
        return result;
    }

    /**
     * 导出excel
     *
     * @param request
     * @param response
     */
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
        // Step.1 组装查询条件
        QueryWrapper<KuaishouDirectionalPackage> queryWrapper = null;
        try {
            String paramsStr = request.getParameter("paramsStr");
            if (oConvertUtils.isNotEmpty(paramsStr)) {
                String deString = URLDecoder.decode(paramsStr, "UTF-8");
                KuaishouDirectionalPackage kuaishouDirectionalPackage = JSON.parseObject(deString, KuaishouDirectionalPackage.class);
                queryWrapper = QueryGenerator.initQueryWrapper(kuaishouDirectionalPackage, request.getParameterMap());
            }
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }

        //Step.2 AutoPoi 导出Excel
        ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
        List<KuaishouDirectionalPackage> pageList = kuaishouDirectionalPackageService.list(queryWrapper);
        //导出文件名称
        mv.addObject(NormalExcelConstants.FILE_NAME, "自动投放-定向包详情列表");
        mv.addObject(NormalExcelConstants.CLASS, KuaishouDirectionalPackage.class);
        mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("自动投放-定向包详情列表数据", "导出人:Jeecg", "导出信息"));
        mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
        return mv;
    }

    /**
     * 通过excel导入数据
     *
     * @param request
     * @param response
     * @return
     */
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
        for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
            MultipartFile file = entity.getValue();
            ImportParams params = new ImportParams();
            params.setTitleRows(2);
            params.setHeadRows(1);
            params.setNeedSave(true);
            try {
                List<KuaishouDirectionalPackage> listKuaishouDirectionalPackages = ExcelImportUtil.importExcel(file.getInputStream(), KuaishouDirectionalPackage.class, params);
                kuaishouDirectionalPackageService.saveBatch(listKuaishouDirectionalPackages);
                return Result.ok("文件导入成功！数据行数:" + listKuaishouDirectionalPackages.size());
            } catch (Exception e) {
                log.error(e.getMessage(), e);
                return Result.error("文件导入失败:" + e.getMessage());
            } finally {
                try {
                    file.getInputStream().close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return Result.ok("文件导入失败！");
    }

}
